/*

LEDtool 1.0 - (c) 2014 by Axel Muhr

A small tool to test the functionality of a connected T2i2c TRAM
under Helios.
To learn more about the T2i2c visit: http://www.geekdot.com/t2i2c.html

Compile with:
c -D_BSD LEDtool.c -o LEDtool

Caution:
You have to have a correct network-setup to use this tool without
crashing your Helios system. This means that the T2i2c TRAM is 
correctly defined in your .map file, for example here's a 
2 Transputer and a T2i2c system:

	processor 00 { ~IO, ~01, , ;}
	processor 01 { , , ~00, ~02, ;}
	processor 02 { , , , ~01 ; native}

i.e. the T2i2c TRAM uses Link #3 to read from Transputer 01 and is defined
'native', so Helios will not try to boot it or check it in any way.
To access the T2i2c TRAM using this example, you have to be on Transputer 01 
and define Link-3 using the '-l' switch.
If your shell is NOT located on Transputer 01, see the manual for the 'remote'
command.

*/

#include <helios.h>
#include <link.h>
#include <codes.h>
#include <nonansi.h>
#include <stdlib.h>
#include <getopt.h>

#define DELAY 1

int MyLink= -1;

int command = 0;

static word	startup(void);
static void	shutdown(void);
void		do_test(int);
void		send_byte(char);
void 		usage(void);


int main(int argc, char **argv) {

int clear_flag = 0;
int test_flag = 0;
int c;

	while ((c = getopt (argc, argv, "xc:tl:h")) != -1)
		switch (c)
           {
       case 'x':
         clear_flag = 1;
         break;
       case 'c':
         command = atoi(optarg);
           if (command <1 || command > 255) {
         	 printf ("Command has to be between 1 and 255.\n");
         	 exit(1);
         	}
         break;
       case 't':
         test_flag = 1;
         break;
       case 'l':
         MyLink = atoi(optarg);
         if (MyLink <0 || MyLink > 3) {
         	 printf ("Link Number has to be between 0 and 3.\n");
         	 exit(1);
         	}
         break;
       case '?':
       case 'h':
       default:
       	 usage();
         exit(0);
    }

if (MyLink < 0) {
    printf ("You have to provide a Link Number (0-3).\n");
    usage();
    exit(1);
}

/* Ok, we have the bare minimum of parameters... */

startup();

if (clear_flag) {
		send_byte(255);
		send_byte(99);
}

if (command != 0) {
		send_byte(255);
		send_byte(command);
}

if (test_flag) {
	do_test(31);	
}

shutdown();	
return(0);
}

void usage() {
	printf("LEDtool v1.0, Transputer link to T2i2c-tool, (c)2014 by Axel Muhr\n \
 Usage: lnktool [x|c #|t|] l # - valid options are:\n \
 -x :  Clear LED \n \
 -c #: Send command given as parameter\n \
 -t :  Send testpattern to LED\n \
 -l #: Transputer Link # to be used (0-3)\n \
\n");
}

void do_test (int z)
{
	BYTE i, muh;
	
	/* Send data in Percent */

	for (i=0; i<z; i++)
    {
	 LinkOut(1 , MyLink,&i, OneSec/DELAY);
	 muh = rand() % 100;
	 LinkOut(1 , MyLink, &muh, OneSec/DELAY);
	 muh = rand() % 100;
	 LinkOut(1 , MyLink, &muh, OneSec/DELAY);
    }

}

static word startup(void)
{ LinkInfo	info;
  LinkConf	conf;

	/* 1) reconfigure link 3 to dumb so that it can be used	*/
  if (LinkData(MyLink, &info) < Err_Null)
   return(EC_Error + EG_Invalid + EO_Link);

  conf.Flags	= info.Flags;
  conf.Id	= info.Id;
  conf.Mode 	= Link_Mode_Dumb;
  conf.State	= Link_State_Dumb;
  if (Configure(conf) < Err_Null)
   return(EC_Error + EG_Broken + EO_Link);

	/* 2) Obtain sole access to the link			*/
  if (AllocLink(MyLink) < Err_Null)
   return(EC_Error + EG_InUse + EO_Link);


  return(Err_Null);
}

static void shutdown(void)
{ 
  FreeLink(MyLink);
}

void linkout_buf(word linkno, void *buf, word size)
{
  LinkOut(size,linkno,buf,OneSec);
}

void send_byte(char b)
{
	LinkOut(1,MyLink,&b,OneSec);
}
